package com.lordjoe.lib.xml;
import com.lordjoe.utilities.*;
import java.util.*;
import java.io.*;
/**
* Object representing a Typelib - 
* created by parsing the file generated by XMLGenerator
* com.lordjoe.lib.xml.TypeLib
* @author Steve Lewis
*/
public class TypeLib implements ITagHandler
{
    public static boolean gNotFirstRead;
    
    private String m_Name;
    private String m_ProgID;
    private String m_File;
    private String m_Guid;
    private int m_MajorVersion;
    private int m_MinorVersion;
    private List m_Enums;
    private List m_Dispatchs;
    private List m_CoClasses;
    private Map m_CoClassMap;
    private Map m_EnumMap;
    
    public static TypeLib readTypeLib(String FileName)
    {
        if(!gNotFirstRead)
            XMLSerializer.registerTag("TypeLib",TypeLib.class);
        TypeLib ret = (TypeLib)XMLSerializer.parseFile(FileName);
        ret.mapCoClasses();
        ret.mapEnums();
        return(ret);
    }
    
    public TypeLib() 
    {
        m_Enums = new ArrayList();
        m_Dispatchs = new ArrayList();
        m_CoClasses = new ArrayList();
    }
    
    public void mapCoClasses() 
    {
        m_CoClassMap = new HashMap();
        COMCOClass[] TheClasses = getCOClasses();
        for(int i = 0; i < TheClasses.length; i++)
            TheClasses[i].register(m_CoClassMap);
    }
    
    public COMCOClass findCOClass(String in)
    {
        return((COMCOClass)m_CoClassMap.get(in));
    }
    
    
    public void mapEnums() 
    {
        m_EnumMap = new HashMap();
        COMEnum[] TheClasses = getEnums();
        for(int i = 0; i < TheClasses.length; i++)
            TheClasses[i].register(m_EnumMap);
    }
    
    public COMEnum findEnum(String in)
    {
        return((COMEnum)m_EnumMap.get(in));
    }
    
    public void addEnum(COMEnum added) 
    {
        m_Enums.add(added);
    }
    
    public void addCOClass(COMCOClass added) 
    {
        m_CoClasses.add(added);
    }
    
    public void addDispatch(COMDispatch added) 
    {
        m_Dispatchs.add(added);
    }
    /**
    * code to get parameter ProgID
    * @return <Add Comment Here>
    * @see getProgID
    */
    public String getProgID()
    {
        return(m_ProgID);
    }

    /**
    * code to set parameter ProgID
    * @param in <Add Comment Here>
    * @see getProgID
    */
    public void setProgID(String in)
    {
        m_ProgID = in;
    }


    /**
    * code to get parameter Name
    * @return <Add Comment Here>
    * @see getName
    */
    public String getName()
    {
        return(m_Name);
    }

    /**
    * code to set parameter Name
    * @param in <Add Comment Here>
    * @see getName
    */
    public void setName(String in)
    {
        m_Name = in;
    }

    /**
    * code to get parameter File
    * @return <Add Comment Here>
    * @see getFile
    */
    public String getFile()
    {
        return(m_File);
    }

    /**
    * code to set parameter File
    * @param in <Add Comment Here>
    * @see getFile
    */
    public void setFile(String in)
    {
        m_File = in;
    }

    /**
    * code to get parameter Guid
    * @return <Add Comment Here>
    * @see getGuid
    */
    public String getGuid()
    {
        return(m_Guid);
    }

    /**
    * code to set parameter Guid
    * @param in <Add Comment Here>
    * @see getGuid
    */
    public void setGuid(String in)
    {
        m_Guid = in;
    }

    /**
    * code to get parameter MajorVersion
    * @return <Add Comment Here>
    * @see getMajorVersion
    */
    public int getMajorVersion()
    {
        return(m_MajorVersion);
    }

    /**
    * code to set parameter MajorVersion
    * @param in <Add Comment Here>
    * @see getMajorVersion
    */
    public void setMajorVersion(int in)
    {
        m_MajorVersion = in;
    }

    /**
    * code to get parameter MinorVersion
    * @return <Add Comment Here>
    * @see getMinorVersion
    */
    public int getMinorVersion()
    {
        return(m_MinorVersion);
    }

    /**
    * code to set parameter MinorVersion
    * @param in <Add Comment Here>
    * @see getMinorVersion
    */
    public void setMinorVersion(int in)
    {
        m_MinorVersion = in;
    }

    /**
    * code to get parameter Enums
    * @return <Add Comment Here>
    * @see getEnums
    */
    public COMEnum[] getEnums()
    {
        COMEnum[] ret = new COMEnum[m_Enums.size()];
        m_Enums.toArray(ret);
        return(ret);
    }


    /**
    * code to get parameter Enums
    * @return <Add Comment Here>
    * @see getEnums
    */
    public COMCOClass[] getCOClasses()
    {
        COMCOClass[] ret = new COMCOClass[m_CoClasses.size()];
        m_CoClasses.toArray(ret);
        return(ret);
    }

    /**
    * code to get parameter Dispatchs
    * @return <Add Comment Here>
    * @see getDispatchs
    */
    public COMDispatch[] getDispatchs()
    {
        COMDispatch[] ret = new COMDispatch[m_Dispatchs.size()];
        m_Dispatchs.toArray(ret);
        return(ret);
    }

    
    /** 
    * This returns the object which will handle the tag - the handler
    * may return itself or create a sub-object to manage the tag
    * @param TagName non-null name of the tag
    * @param attributes non-null array of name-value pairs
    * @return possibly null handler
    */
    public Object handleTag(String TagName,NameValue[] attributes)
    {
        if(TagName.equals("enum")) {
            COMEnum ret = new COMEnum();
            addEnum(ret);
            return(ret);
        }
        if(TagName.equals("Dispatch")) {
            COMDispatch ret = new COMDispatch();
            addDispatch(ret);
            return(ret);
        }
        if(TagName.equals("CoClass")) {
            COMCOClass ret = new COMCOClass();
            addCOClass(ret);
            return(ret);
        }
        return(null); // not understood
    }
    
    public static void main(String[] args) throws IOException
    {
        if(args.length < 2) {
            System.out.println("Usage:");
            System.out.println("java com.lordjoe.lib.xml.TypeLib <xml file> <package>");
            System.out.println("i.e. java com.lordjoe.lib.xml.TypeLib Outlook_Application.xml com.microsoft.outlook");
        }
        TypeLib TheLib = readTypeLib(args[0]);
        String APackage = args[1]; // "com.microsoft.excel";
     
        TypelibEnumBuilder TheBuilder = new TypelibEnumBuilder(APackage,"");
        
        TheBuilder.buildCOMEnums(TheLib.getEnums());
        TypelibDispatchBuilder TheDBuilder = new TypelibDispatchBuilder(TheLib,APackage,"");
        TheDBuilder.buildCOMSurrogates(TheLib.getDispatchs());
        TheLib = null;
    }
    
}
