package com.lordjoe.utilities;
import com.lordjoe.lib.xml.*;
import java.io.*;
import java.util.*;
import java.text.*;
import java.lang.reflect.*;

/** 
* Sample Use
*     public void buildInterfaces()
*    {
*        try {
*            COMEnumBuilder TheBuilder = new COMEnumBuilder();
*            TheBuilder.buildCOMEnums("com.onewebsystems.eagle.outlook","Enum","com/microsoft/outlook");
*       }
*        catch(java.lang.Exception ex) {
*            ex.printStackTrace();
*        }
*    }
* 
*  Updated to make it compile on Java 1.5
*  Miika Nurminen
*  
*    @date> Fri 5. May 2006
*    @version> 1.1
*/
public class COMEnumBuilder
{
    public COMEnumBuilder()
    {
    }
    
    
    public void buildCOMEnums(String Package,String[] Prefixs,String Suffix,String in) throws IOException
    {
        File TheFile = new File(in);
        if(!TheFile.exists())
            throw new IllegalArgumentException("File " + in + " does not exits");
        if(!TheFile.isDirectory())
            throw new IllegalArgumentException("File " + in + " is not a directory");
        String[] items = FileUtilities.getAllFilesWithExtension(in,"java");
        String TestString = null;
        String Prefix = null;
        for(int i = 0; i < items.length; i++) {
            for(int k = 0; k < Prefixs.length; i++) {
                TestString = in + "/" + Prefixs[i];
                if(items[i].startsWith(TestString)) {
                    Prefix = Prefixs[i];
                    break;
                }
            }
            if(Prefix != null) {
                String ClassName = items[i].substring(0,items[i].length() - 5);
                ClassName = ClassName.replace('\\','/');
                ClassName = ClassName.replace('/','.');
                Class test = ClassUtilities.classForName(ClassName);
                if(test != null) {
                    if(test.isInterface()) 
                        buildCOMEnum(Package,Prefix,Suffix,test,null);
                }
                    
            }
        }
    }
    
    
    public void buildCOMEnum(String Package,String Prefix,String Suffix,Class in,File TargetDirectory) throws IOException
    {
        if(!in.isInterface())
            throw new IllegalArgumentException("buildCOMEnum reguires an interface");
        String inName = ClassUtilities.shortClassName(in);
        String ClassName = inName.substring(Prefix.length()) + Suffix;
        String FileName = Package + "." + ClassName;
        FileName = FileName.replace('.','/');
        FileName = FileName.replace('\\','/');
        FileName += ".java";
        if(TargetDirectory != null)
            FileName = TargetDirectory.getAbsolutePath().replace('\\','/') + "/" + FileName;
        FileUtilities.guaranteeDirectory(FileName);
        File Test = new File(FileName);
 //       if(Test.exists())
 //           return; // do not overwrite
        PrintWriter out = new PrintWriter(new FileOutputStream(Test));
        buildCOMEnum(Package,Prefix,ClassName,out,in);
    }
    
    protected void buildCOMEnum(String Package,String Prefix,String ClassName,PrintWriter out,Class in) throws IOException
    {
        writeHeader(Package,ClassName,out,in);
        writeInitializer(ClassName,Prefix,out,in);
        writeAccessor(ClassName,out,in);
        writeConstructor(ClassName,out,in);
        writeFooter(ClassName,out,in);
        out.close();
    }
    protected void writeInitializer(String ClassName,String Prefix,PrintWriter out,Class in) throws IOException
    {
        int indent = 1;
        Util.indent(out,indent);
        out.println("static {");
        Field[] fields = in.getFields();
        for(int i = 0; i < fields.length; i++) 
            writeFieldItem(ClassName,Prefix,out,fields[i],indent + 1,i == 0);
        Util.indent(out,indent);
        out.println("}");
        out.println();
    }
    
    protected void writeFieldItem(String ClassName,String Prefix,PrintWriter out,Field in,int indent,boolean isSample) throws IOException
    {
        int modifiers = in.getModifiers();
        
        if(!Modifier.isStatic(modifiers) || !Modifier.isFinal(modifiers))
            return;
        String ItemName = in.getName();
        if(ItemName.toLowerCase().startsWith(Prefix.toLowerCase()))
            ItemName = ItemName.substring(Prefix.length());
        Util.indent(out,indent);
        if(isSample) 
            out.print("gSampleOption = ");
        out.print("new " + ClassName + "(\"");
        out.print(ItemName + "\"," + in.getName());
        out.println(");");
    }
    
    protected void writeAccessor(String ClassName,PrintWriter out,Class in) throws IOException
    {
        int indent = 1;
        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* look up one instance by namee");
        Util.indent(out,indent);
        out.println("* @return non-null instance");
        Util.indent(out,indent);
        out.println("* @throws IllegalArgumentException if the name is not a member");
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("public static " + ClassName + " getOption(String in) {");
        Util.indent(out,indent + 1);
        out.println("Object ret = gOptions.get(in);");
        Util.indent(out,indent + 1);
        out.println("if(ret == null)");
        Util.indent(out,indent + 2);
        out.println("throw new IllegalArgumentException(\"Comparison \" + in + \" not found - valid comparisons are:\\n\" + ");
        Util.indent(out,indent + 3);
        out.println("Util.buildListString(Util.getKeyStrings(gOptions),0));");
        Util.indent(out,indent + 1);
        out.println("return((" + ClassName + ")ret);");
        Util.indent(out,indent);
        out.println("}");
        out.println();


        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* look up one instance by namee");
        Util.indent(out,indent);
        out.println("* @return non-null instance");
        Util.indent(out,indent);
        out.println("* @throws IllegalArgumentException if the name is not a member");
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("public static " + ClassName + " getOptionByValue(in in) {");
        Util.indent(out,indent + 1);
        out.println("Object ret = gOptionsByValue.get(in);");
        Util.indent(out,indent + 1);
        out.println("if(ret == null)");
        Util.indent(out,indent + 2);
        out.println("throw new IllegalArgumentException(\"Comparison \" + in + \" not found - valid comparisons are:\\n\" + ");
        Util.indent(out,indent + 3);
        out.println("Util.buildListString(Util.getValueStrings(gOptionsByValue),0));");
        Util.indent(out,indent + 1);
        out.println("return((" + ClassName + ")ret);");
        Util.indent(out,indent);
        out.println("}");
        out.println();

        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* return the names of all options for this enum type");
        Util.indent(out,indent);
        out.println("* @return non-null array of names");
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("public static String[] optionNames() {");
        Util.indent(out,indent + 1);
        out.println("return(gSampleOption.getOptionNames());");
        Util.indent(out,indent);
        out.println("}");
        out.println();

        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* return a random instance");
        Util.indent(out,indent);
        out.println("* @return non-null instance");
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("public static " + ClassName + " randomElement() {");
        Util.indent(out,indent + 1);
        out.println("return((" + ClassName + ")gSampleOption.randomType());");
        Util.indent(out,indent);
        out.println("}");
        out.println();
    }
   
    protected void writeConstructor(String ClassName,PrintWriter out,Class in) throws IOException
    {
        int indent = 1;
        Util.indent(out,indent);
        out.println("private " + ClassName + "(String name,int value) {");
        Util.indent(out,indent + 1);
        out.println("super(name,value);");
        Util.indent(out,indent);
        out.println("}");
        out.println();
    }
    
    
    protected void writeHeader(String Package,String ClassName,PrintWriter out,Class in) throws IOException
    {
        out.println("package " + Package + ";");
        out.println("import " + in.getName() + ";");
        out.println("import com.lordjoe.utilities.*;" );
        out.println("import java.util.HashMap;" );
        out.println("import java.util.Map;" );
        out.println("import java.util.HashMap;" );
        out.println("import java.io.Serializable;" );
        
        out.println();
        out.println("/**");
        out.println("* java enum derived from " + in.getName());
        out.println("* autogenerated by COMUtilities.buildCOMEnum ");
        out.println("* DO NOT EDIT");
        out.println("* @author S Lewis");
        out.println("*/");
        out.println("public class " + ClassName + " extends EnumeratedInt implements Serializable," + in.getName());
        out.println("{");
        out.println();
        Util.indent(out,1);
        out.println("// one option exposed for convenience");
        Util.indent(out,1);
        out.println("protected static " + ClassName + " gSampleOption;");
        out.println();
        Util.indent(out,1);
        out.println("// Map - key is name value ins Enum");
        Util.indent(out,1);
        out.println("protected static Map gOptions = new HashMap();");
        Util.indent(out,1);
        out.println("protected static Map gOptionsByValue = new HashMap();");
        out.println();
    }
    
    protected void writeFooter(String ClassName,PrintWriter out,Class in) throws IOException
    {
        int indent = 1;
        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* return a Map of possible values");
        Util.indent(out,indent);
        out.println("* @return non-null name");
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("protected Map getOptionMap() {");
        Util.indent(out,indent + 1);
        out.println("return(gOptions);");
        Util.indent(out,indent);
        out.println("}");
        out.println();

        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* return a Map of possible values");
        Util.indent(out,indent);
        out.println("* @return non-null name");
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("protected Map getValueMap() {");
        Util.indent(out,indent + 1);
        out.println("return(gOptionsByValue);");
        Util.indent(out,indent);
        out.println("}");
        out.println();
        out.println("}");
    }
}
