package com.lordjoe.utilities;
import java.util.*;

public class COMFilter
{
    public static String valueToString(Object value) 
    {
        if(value instanceof Double)
            return(value.toString());
        if(value instanceof Integer)
            return(value.toString());
        if(value instanceof Date)
            return(COMUtilities.vbFormatDate((Date)value));
        if(value instanceof Calendar)
            return(COMUtilities.vbFormatDate((Calendar)value));
        return("'" + value.toString() + "'");
    }

    private List m_Conditions;
    
    public COMFilter() {
        m_Conditions = new ArrayList();
    }
    
    public void addAndCondition(COMFilter filter)
    {
        m_Conditions.add(new AndFilterCondition(filter));
    }
    
    public void addOrCondition(COMFilter filter)
    {
        m_Conditions.add(new OrFilterCondition(filter));
    }
    
    public void addAndCondition(String Property,String condition,int Value)
    {
        addAndCondition(Property, condition,new Integer(Value));
    }
    
    public void addAndCondition(String Property,String condition,double Value)
    {
        addAndCondition(Property, condition,new Double(Value));
    }
    
    public void addAndCondition(String Property,String condition,Object Value)
    {
        m_Conditions.add(new SpecificAndCondition(Property, condition, Value));
    }
    
    public void addOrCondition(String Property,String condition,Object Value)
    {
        m_Conditions.add(new SpecificOrCondition(Property, condition, Value));
    }
    
    public void addOrCondition(String Property,String condition,int Value)
    {
        addOrCondition(Property, condition,new Integer(Value));
    }
    
    public void addOrCondition(String Property,String condition,double Value)
    {
        addOrCondition(Property, condition,new Double(Value));
    }
    
    public String toString()
    {
        int count = m_Conditions.size();
        if(count == 0)
            return("");
        StringBuffer sb = new StringBuffer();
        sb.append(m_Conditions.get(0).toString());
        for(int i = 1; i < count; i++ ) {
            Object test = m_Conditions.get(i);
            if(test instanceof AndCondition)
                sb.append(" and ");
            if(test instanceof OrCondition)
                sb.append(" or ");
            sb.append(test.toString());
        }
        return(sb.toString());
    }
    
    protected static abstract class AndCondition
    {
    }
    
    protected static abstract class OrCondition
    {
    }
    
    protected static class OrFilterCondition extends OrCondition
    {
        private COMFilter m_Filter;
        protected OrFilterCondition(COMFilter filter)
        {
            m_Filter = filter;
        }
        public String toString()
        {
            String myString = m_Filter.toString();
            return("(" + myString + ")");
        }
    }
    
    protected static class AndFilterCondition extends AndCondition
    {
        private COMFilter m_Filter;
        protected AndFilterCondition(COMFilter filter)
        {
            m_Filter = filter;
        }
        public String toString()
        {
            String myString = m_Filter.toString();
            return("(" + myString + ")");
        }
    }
    
    
    protected static class SpecificAndCondition extends AndCondition
    {
        private String m_Property;
        private COMComparison m_condition;
        private Object m_Value;
        
        protected SpecificAndCondition(String Property,String condition,Object Value)
        {
            m_Property = Property;
            m_condition = COMComparison.getComparison(condition);
            m_Value = Value;
        }
        
        public String toString()
        {
            return("[" + m_Property + "] " + m_condition + " " + valueToString(m_Value));
        }
    }
    
    
    protected static class SpecificOrCondition extends OrCondition
    {
        private String m_Property;
        private COMComparison m_condition;
        private Object m_Value;
        
        protected SpecificOrCondition(String Property,String condition,Object Value)
        {
            m_Property = Property;
            m_condition = COMComparison.getComparison(condition);
            m_Value = Value;
        }
        
        public String toString()
        {
            return("[" + m_Property + "] " + m_condition + " " + valueToString(m_Value));
        }
    }
}
