package com.lordjoe.utilities;
import com.lordjoe.lib.xml.*;
import java.io.*;
import java.util.*;
import java.text.*;
import java.lang.reflect.*;

public class COMUtilities
{
    protected static Set gAllowedCOMCasts = new HashSet();
    
    /**
    * force specified ComObject to be cast 
    * @param Target non-null class of target object accepting the cast
    * @param Source non-null class of source object being cast
    * @return true if cast is OK
    */
    public static boolean allowCOMCast(Class Target,Class Source)
    {
        return(gAllowedCOMCasts.contains(new ComCast(Target,Source)));
    }
    
    /**
    * register an allowed case
    * @param Target non-null class of target object accepting the cast
    * @param Source non-null class of source object being cast
    */
    public static void registerCOMCast(Class Target,Class Source)
    {
        gAllowedCOMCasts.add(new ComCast(Target,Source));
    }
    
    public static class ComCast
    {
        public Class target;
        public Class source;
        public ComCast(Class atarget,Class asource) {
            target = atarget;
            source = asource;
        }
        
        public boolean equals(Object in) {
            if(!(in instanceof ComCast)) 
                return(false);
            ComCast test = (ComCast)in;
            return(test.target == target && test.source == source);
        }
        
        public int hashCode() {
            return(target.hashCode() ^ source.hashCode());
        }
        
    }
    
    /** 
    * Convert a COM collection impementing GetFirst,GetNext and
    * get_Count into an array of object of class Target
    * @param Collection possibly-null COM object as above
    * @param Target non-null class with a constructor (Object)
    * @return non-null array
    */
    public static Object[] getItemsArray(Object Collection,Class Target)
    {
        List holder = new ArrayList();
        if(Collection != null ) {
            Iterator it = new ReflectiveIterator(Collection,Target);
            while(it.hasNext()) 
                holder.add(it.next());
        }
        return(Util.collectionToArray(holder,Target));
    }
 
    /** 
    * Convert a COM collection impementing GetFirst,GetNext and
    * get_Count into an array of object of class Target
    * @param Collection possibly-null COM object as above
    * @param Target non-null class with a constructor (Object)
    * @param start non-negative start position
    * @param end non-negative end position - note 0 means ignore start, show all
    * @return non-null array
    */
    public static Object[] getItemsArray(Object Collection,Class Target,int start,int end)
    {
        if(end == 0) 
            return(getItemsArray(Collection,Target));
        List holder = new ArrayList();
        if(Collection != null ) {
            Iterator it = new ReflectiveIterator(Collection,Target,start,end);
            while(it.hasNext()) 
                holder.add(it.next());
        }
        return(Util.collectionToArray(holder,Target));
    }

    /**
    * convert to the date format used by Outlook and Excell
    * to a Java Calandar
    * @param non-null Date representing the date
    * @return date used by outlook
    */
    public static String vbFormatDate(Date d)
    {
        SimpleDateFormat format = new SimpleDateFormat("MM/dd/yyyy HH:mm");
        return("'" + format.format(d) + "'");
    }
    
    /**
    * convert to the date format used by Outlook and Excell
    * to a Java Calandar
    * @param non-null Calendar representing the date
    * @return date used by outlook
    */
    public static String vbFormatDate(Calendar c)
    {
        return(vbFormatDate(c.getTime()));
    }

    /**
    * convert to the date format used by Outlook and Excell
    * to a Java Calandar
    * @param non-null Date representing the date
    * @return date used by outlook
    */
    public static double vbDate(Date d)
    {
        Calendar c = new GregorianCalendar();
        c.setTime(d);
        return(vbDate(c));
    }
    
    /**
    * convert to the date format used by Outlook and Excell
    * to a Java Calandar
    * @param non-null Calendar representing the date
    * @return date used by outlook
    */
    public static double vbDate(Calendar c)
    {
        Calendar base = new GregorianCalendar(1899,11,30);
        long diff = c.getTime().getTime() - base.getTime().getTime();
        double ret = diff/ (24 * 60 * 60 * 1000.0);
        return(ret);
    }
    
    /**
    * convert fron the date format used by Outlook and Excell
    * to a Java Calandar
    * @param in - the outlook date
    * @return non-null Calendar representing the date
    */
    public static Calendar fromVBDate(double in) 
    {
        Calendar ret = new GregorianCalendar(1899,11,30);
        int days = (int)in;
        in -= days;
        ret.add(Calendar.DATE,days);
        in *= 24;
        int hours = (int)in;
        ret.add(Calendar.HOUR,hours);
        in -= hours;
        in *= 60;
        in += 0.5; // round
        int min = (int)in;
        ret.add(Calendar.MINUTE,min);
        return(ret);
    }

    /**
    * return true id Test is a key in Values and represents
    * a non-empty string
    * @param Values non-null Map
    * @param Test non-null Test String
    * @return as above
    */
    public static boolean hasProperty(Map Values,String Test)
    {
        Object val = Values.get(Test);
        if(val == null)
            return(false);
        return(val.toString().length() > 0);
    }
    
}
