package com.lordjoe.utilities;
import java.util.*;
import java.io.*;

/**
* enumarated type for comparisons used by VB
*/
public abstract class EnumeratedType
{
     private final String m_Name;
     
     protected EnumeratedType(String name)
     {
        if(name == null) 
            throw new IllegalArgumentException("name cannot be null");
        m_Name = name;
        rememberName();
     }
     
    protected void rememberName()
    {
        getOptionMap().put(getName(),this);
    }
    
    protected void rememberValue()
    {
        getValueMap().put(getEnumeratedData(),this);
    }
        
    /**
    * return the name of the object
    * @return non-null name
    */
    public String getName()
    {
        return(m_Name);
    }
    
     
    /**
    * return a Map of possible values
    * @return non-null name
    */
    protected abstract Map getOptionMap();
     
    /**
    * return a Map of possible values
    * wheer the key is the value
    * @return non-null name
    */
    protected abstract Map getValueMap();
    
     
    /**
    * return the class of the object held by the 
    * type 
    * @return non-null name
    */
    public String[] getOptionNames()
    {
        return(Util.getKeyStrings(getOptionMap()));
    }
    
    /**
    * allows the objects to be serialized
    * and returns the proper enumerated element
    * @return non-null object
    * @throws NullPointerException on error
    */
    protected Object readResolve() throws ObjectStreamException
    {
        return(getEnumeratedTypeByValue(getEnumeratedData()));
    }
    
    /**
    * return the class of the object held by the 
    * type 
    * @return non-null name
    */
    public EnumeratedType getEnumeratedTypeByValue(Object value)
    {
        return((EnumeratedType)getValueMap().get(value));
    }
     
    public EnumeratedType randomType()
    {
        return((EnumeratedType)Util.randomValue(getOptionMap()));
    }
     
    /**
    * return the class of the object held by the 
    * type 
    * @return non-null class
    */
    public abstract Class getEnumeratedClass();
          
    /**
    * return the data underlying the enumeration
    * @return non-null class
    */
    public abstract Object getEnumeratedData();

    /**
    * I gues the object should be the toString
    * @return non-null String
    */
    public String toString() {
        return(getEnumeratedData().toString());
    }
      
}
