package com.lordjoe.utilities;
import com.lordjoe.lib.xml.*;
import java.io.*;
import java.util.*;
import java.text.*;
import java.lang.reflect.*;

/** 
* Sample Use
*     public void buildInterfaces()
*    {
*        try {
*            COMEnumBuilder TheBuilder = new TypelibEnumBuilder("com.microsoft.excel","import com.microsoft.office.*;\n");
*            TheBuilder.buildCOMEnums("com.onewebsystems.eagle.outlook","Enum","com/microsoft/outlook");
*       }
*        catch(java.lang.Exception ex) {
*            ex.printStackTrace();
*        }
*    }
*  Updated to make it compile on Java 1.5
*  Miika Nurminen
*  
*    @date> Fri 5. May 2006
*    @version> 1.1
* 
*/
public class TypelibEnumBuilder
{
    public static final String ENUM_SUFFIX = "Enum";
    
    private String m_Package;
    private String m_AddedImports;
    private File m_TargetDirectory;
    
    public TypelibEnumBuilder()
    {
        m_AddedImports = "";
        m_Package = "";
    }
    public TypelibEnumBuilder(String APackage,String AddedImports)
    {
        this();
        m_AddedImports = AddedImports;
        m_Package = APackage;
    }
    /**
    * code to get parameter TargetDirectory
    * @return <Add Comment Here>
    * @see getTargetDirectory
    */
    public File getTargetDirectory()
    {
        return(m_TargetDirectory);
    }

    /**
    * code to set parameter TargetDirectory
    * @param in <Add Comment Here>
    * @see getTargetDirectory
    */
    public void setTargetDirectory(File in)
    {
        m_TargetDirectory = in;
    }

    
     /**
    * code to get parameter Package
    * @return <Add Comment Here>
    * @see getPackage
    */
    public String getPackage()
    {
        return(m_Package);
    }

    /**
    * code to set parameter Package
    * @param in <Add Comment Here>
    * @see getPackage
    */
    public void setPackage(String in)
    {
        m_Package = in;
    }

    /**
    * code to get parameter AddedImports
    * @return <Add Comment Here>
    * @see getAddedImports
    */
    public String getAddedImports()
    {
        return(m_AddedImports);
    }

    /**
    * code to set parameter AddedImports
    * @param in <Add Comment Here>
    * @see getAddedImports
    */
    public void setAddedImports(String in)
    {
        m_AddedImports = in;
    }

    public void buildCOMEnums(COMEnum[] Items) throws IOException
    {
        for(int i = 0; i < Items.length; i++) 
            buildCOMEnum(Items[i]);
    }
    
    
    
    protected void buildCOMEnum(COMEnum in) throws IOException
    {
        String ClassName = in.getName() + ENUM_SUFFIX;
        String FileName = null;
        if(getPackage().length() > 0)
            FileName = getPackage() + ".com_enum." + ClassName;
        else
            FileName =  ClassName;
            
        FileName = FileName.replace('.','/');
        FileName = FileName.replace('\\','/');
        FileName += ".java";
        if(getTargetDirectory() != null)
            FileName = getTargetDirectory().getAbsolutePath().replace('\\','/') + "/" + FileName;
        FileUtilities.guaranteeDirectory(FileName);
        File Test = new File(FileName);
 //       if(Test.exists())
 //           return; // do not overwrite
        PrintWriter out = new PrintWriter(new FileOutputStream(Test));
        writeHeader(ClassName,out,in);
    //    writeInitializer(ClassName,out,in);
        writeAccessor(ClassName,out,in);
        writeConstructor(ClassName,out,in);
        writeFooter(ClassName,out,in);
        out.close();
    }
    protected void writeInitializer(String ClassName,PrintWriter out,COMEnum in) throws IOException
    {
        int indent = 1;
        Util.indent(out,indent);
        out.println("static {");
        NameValue[] fields = in.getitems();
        for(int i = 0; i < fields.length; i++) 
            writeFieldItem(ClassName,out,fields[i],indent + 1);
        Util.indent(out,indent);
        out.println("}");
        out.println();
    }
    
    protected void writeFieldItem(String ClassName,PrintWriter out,NameValue in,int indent) throws IOException
    {
        String ItemName = in.m_Name;
        if(ItemName.endsWith("_A")) // in CDO these entries are redundent with entries without _A
            return;
        Util.indent(out,indent);
        out.print("public static final " + ClassName + " " + buildFieldName(in) + " = new " + ClassName + "(\"");
        out.print(ItemName + "\"," + in.m_Value);
        out.println(");");
        return;
    }
    
    protected void writeAccessor(String ClassName,PrintWriter out,COMEnum in) throws IOException
    {
        int indent = 1;
        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* look up one instance by namee");
        Util.indent(out,indent);
        out.println("* @return non-null instance");
        Util.indent(out,indent);
        out.println("* @throws IllegalArgumentException if the name is not a member");
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("public static " + ClassName + " getOption(String in) {");
        Util.indent(out,indent + 1);
        out.println("Object ret = gOptions.get(in);");
        Util.indent(out,indent + 1);
        out.println("if(ret == null)");
        Util.indent(out,indent + 2);
        out.println("throw new IllegalArgumentException(\"Comparison \" + in + \" not found - valid comparisons are:\\n\" + ");
        Util.indent(out,indent + 3);
        out.println("Util.buildListString(Util.getKeyStrings(gOptions),0));");
 
        Util.indent(out,indent + 1);
        out.println("return((" + ClassName + ")ret);");
        Util.indent(out,indent);
        out.println("}");
        out.println();

        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* look up one instance by name");
        Util.indent(out,indent);
        out.println("* @return possibly-null instance");
/*        
        Util.indent(out,indent);
        out.println("* @throws IllegalArgumentException if the name is not a member");
 */
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("public static " + ClassName + " getOptionByValue(int in) {");
        Util.indent(out,indent + 1);
        out.println("Object ret = gOptionsByValue.get(new Integer(in));");
/*
        Util.indent(out,indent + 1);
        out.println("if(ret == null)");
        Util.indent(out,indent + 2);
        out.println("throw new IllegalArgumentException(\"Comparison \" + in + \" not found - valid comparisons are:\\n\" + ");
        Util.indent(out,indent + 3);
        out.println("Util.buildListString(Util.getValueStrings(gOptionsByValue),0));");
        Util.indent(out,indent + 1);
 */
        out.println("return((" + ClassName + ")ret);");
        Util.indent(out,indent);
        out.println("}");
        out.println();

        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* return the names of all options for this enum type");
        Util.indent(out,indent);
        out.println("* @return non-null array of names");
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("public static String[] optionNames() {");
        Util.indent(out,indent + 1);
        out.println("return(gSampleOption.getOptionNames());");
        Util.indent(out,indent);
        out.println("}");
        out.println();

        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* return a random instance");
        Util.indent(out,indent);
        out.println("* @return non-null instance");
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("public static " + ClassName + " randomElement() {");
        Util.indent(out,indent + 1);
        out.println("return((" + ClassName + ")gSampleOption.randomType());");
        Util.indent(out,indent);
        out.println("}");
        out.println();
    }
   
    protected void writeConstructor(String ClassName,PrintWriter out,COMEnum in) throws IOException
    {
        int indent = 1;
        Util.indent(out,indent);
        out.println("private " + ClassName + "(String name,int value) {");
        Util.indent(out,indent + 1);
        out.println("super(name,value);");
        Util.indent(out,indent);
        out.println("}");
        out.println();
    }
    
    
    protected void writeHeader(String ClassName,PrintWriter out,COMEnum in) throws IOException
    {
        out.println("package " + getPackage() + ".com_enum;");
        out.println("import com.lordjoe.utilities.*;" );
        if(getAddedImports().length() > 0)
            out.println(getAddedImports());
        out.println("import java.util.Map;" );
        out.println("import java.util.HashMap;" );
        out.println("import java.io.Serializable;" );
        
        out.println();
        out.println("/**");
        out.println("* java enum derived from " + in.getName());
        out.println("* autogenerated by COMUtilities.buildCOMEnum ");
        out.println("* DO NOT EDIT");
        out.println("* @author S Lewis");
        out.println("*/");
        out.println("public class " + ClassName + " extends EnumeratedInt implements Serializable" );
        out.println("{");
        out.println();
        Util.indent(out,1);
        out.println("// Map - key is name value ins Enum");
        Util.indent(out,1);
        out.println("protected static Map gOptions = new HashMap();");
        Util.indent(out,1);
        out.println("protected static Map gOptionsByValue = new HashMap();");
        out.println();
        NameValue[] fields = in.getitems();
        for(int i = 0; i < fields.length; i++) 
            writeFieldItem(ClassName,out,fields[i],1);
        Util.indent(out,1);
        out.println("// one option exposed for convenience");
        Util.indent(out,1);
        out.println("protected static final " + ClassName + " gSampleOption = " + buildFieldName(fields[0]) + ";");
        out.println();
    }
    
    protected String buildFieldName(NameValue in)
    {
        return(in.m_Name + "_ENUM");
    }
    
    protected void writeFooter(String ClassName,PrintWriter out,COMEnum in) throws IOException
    {
        int indent = 1;
        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* return a Map of possible values");
        Util.indent(out,indent);
        out.println("* @return non-null name");
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("protected Map getOptionMap() {");
        Util.indent(out,indent + 1);
        out.println("return(gOptions);");
        Util.indent(out,indent);
        out.println("}");
        out.println();

        Util.indent(out,indent);
        out.println("/**");
        Util.indent(out,indent);
        out.println("* return a Map of possible values");
        Util.indent(out,indent);
        out.println("* @return non-null name");
        Util.indent(out,indent);
        out.println("*/");
        Util.indent(out,indent);
        out.println("protected Map getValueMap() {");
        Util.indent(out,indent + 1);
        out.println("return(gOptionsByValue);");
        Util.indent(out,indent);
        out.println("}");
        out.println();
        out.println("}");
    }
}
