
#include "stdafx.h"
#include <objbase.h>
#include "COMUtils.h"
// Win32 support for Ole Automation
#include <wchar.h>
#include <string.h>
#include <atlbase.h>
#include <oleauto.h>
#include <olectl.h>
#include "util.h"

extern "C" 
{
static void copyStringChars(JNIEnv *env,jstring  _in,OLECHAR *bsProgId);
static int oleStrLen(const OLECHAR *in);
const int MAX_CHARS = 512;

static void copyStringChars(JNIEnv *env,jstring  _in,OLECHAR *bsProgId)
{
  jboolean isCopy = false;
  const jchar *chars = env->GetStringChars(_in,&isCopy);
  const jchar *src = chars;
  OLECHAR *dst= bsProgId;
  while(*src) {
	  *(dst++) = *(src++);
  }
  *dst = 0;
  env->ReleaseStringChars(_in,chars);
}

static int oleStrLen(const OLECHAR *in)
{
   int ret = 0;
   while(*(in++))
	  ret++;
   return(ret);
}

/*
 * Class:     com_jacob_com_COMUtils
 * Method:    doCLSIDFromProgID
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_jacob_com_COMUtils_doCLSIDFromProgID
  (JNIEnv *env, jclass, jstring  _in)
{
  CLSID clsid;
  HRESULT hr;
  OLECHAR bsProgId[MAX_CHARS]; 	
  OLECHAR StringBase[MAX_CHARS]; 	
  LPOLESTR lpsz = StringBase;
  jstring ret;

  copyStringChars(env,_in,bsProgId);

  // Now, try to find an IDispatch interface for progid
  hr = CLSIDFromProgID(bsProgId, &clsid);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't get object clsid from progid", hr);
    return NULL;
  }
  hr = StringFromGUID2(clsid,lpsz,MAX_CHARS);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't convert GUID to String", hr);
    return NULL;
  }

    // Handle string...
   ret = env->NewString(lpsz,oleStrLen(lpsz));
  return(ret);
}
/*
 * Class:     com_jacob_com_COMUtils
 * Method:    doCLSIDFromString
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_jacob_com_COMUtils_doCLSIDFromString
  (JNIEnv *env, jclass, jstring  _in)
{
  CLSID clsid;
  HRESULT hr;
  OLECHAR bsProgId[MAX_CHARS]; 	
  OLECHAR StringBase[MAX_CHARS]; 	
  LPOLESTR lpsz = StringBase;
  jstring ret;
  // Now, try to find an IDispatch interface for progid
  copyStringChars(env,_in,bsProgId);

  // Now, try to find an IDispatch interface for progid
  hr = CLSIDFromProgID(bsProgId, &clsid);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't get object clsid from progid", hr);
    return NULL;
  }
  hr = StringFromGUID2(clsid,lpsz,MAX_CHARS);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't convert GUID to String", hr);
    return NULL;
  }

    // Handle string...
   ret = env->NewString(lpsz,oleStrLen(lpsz));
  return(ret);
}

/*
 * Class:     com_jacob_com_COMUtils
 * Method:    doIIDFromString
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_jacob_com_COMUtils_doIIDFromString
  (JNIEnv *env, jclass, jstring  _in)
{
  CLSID clsid;
  HRESULT hr;
  OLECHAR bsProgId[MAX_CHARS]; 	
  OLECHAR StringBase[MAX_CHARS]; 	
  LPOLESTR lpsz = StringBase;
  jstring ret;
  // Now, try to find an IDispatch interface for progid
  copyStringChars(env,_in,bsProgId);

  // Now, try to find an IDispatch interface for progid
  hr = IIDFromString(bsProgId, &clsid);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't get object clsid from progid", hr);
    return NULL;
  }
  hr = StringFromGUID2(clsid,lpsz,MAX_CHARS);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't convert GUID to String", hr);
    return NULL;
  }

    // Handle string...
   ret = env->NewString(lpsz,oleStrLen(lpsz));
  return(ret);
}

/*
 * Class:     com_jacob_com_COMUtils
 * Method:    doProgIDFromCLSID
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_jacob_com_COMUtils_doProgIDFromCLSID
  (JNIEnv *env, jclass, jstring  _in)
{
  CLSID clsid;
  HRESULT hr;
  OLECHAR bsProgId[MAX_CHARS]; 	
  OLECHAR StringBase[MAX_CHARS]; 	
  LPOLESTR lpsz = StringBase;
  jstring ret;
  // Now, try to find an IDispatch interface for progid
  copyStringChars(env,_in,bsProgId);

  // Now, try to find an IDispatch interface for progid
  hr = CLSIDFromString(bsProgId, &clsid);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't get object clsid from progid", hr);
    return NULL;
  }


  hr = ProgIDFromCLSID(clsid,&lpsz);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't convert GUID to String", hr);
    return NULL;
  }

    // Handle string...
   ret = env->NewString(lpsz,oleStrLen(lpsz));
  return(ret);
}
/*
 * Class:     com_jacob_com_COMUtils
 * Method:    doStringFromCLSID
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_jacob_com_COMUtils_doStringFromCLSID
  (JNIEnv *env, jclass, jstring  _in)
{
  CLSID clsid;
  HRESULT hr;
  OLECHAR bsProgId[MAX_CHARS]; 	
  OLECHAR StringBase[MAX_CHARS]; 	
  LPOLESTR lpsz = StringBase;
  jstring ret;
  // Now, try to find an IDispatch interface for progid
  copyStringChars(env,_in,bsProgId);

  // Now, try to find an IDispatch interface for progid
  hr = CLSIDFromString(bsProgId, &clsid);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't get object clsid from progid", hr);
    return NULL;
  }

  hr = StringFromCLSID(clsid,&lpsz);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't convert GUID to String", hr);
    return NULL;
  }

    // Handle string...
   ret = env->NewString(lpsz,oleStrLen(lpsz));
  return(ret);
}
/*
 * Class:     com_jacob_com_COMUtils
 * Method:    doStringFromGUID2
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_jacob_com_COMUtils_doStringFromGUID2
  (JNIEnv *env, jclass, jstring  _in)
{
  CLSID clsid;
  HRESULT hr;
  OLECHAR bsProgId[MAX_CHARS]; 	
  OLECHAR StringBase[MAX_CHARS]; 	
  LPOLESTR lpsz = StringBase;
  jstring ret;
  // Now, try to find an IDispatch interface for progid
  copyStringChars(env,_in,bsProgId);

  // Now, try to find an IDispatch interface for progid
  hr = CLSIDFromString(bsProgId, &clsid);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't get object clsid from progid", hr);
    return NULL;
  }

  hr = StringFromGUID2(clsid,lpsz,MAX_CHARS);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't convert GUID to String", hr);
    return NULL;
  }

    // Handle string...
   ret = env->NewString(lpsz,oleStrLen(lpsz));
  return(ret);
}


/*
 * Class:     com_jacob_com_COMUtils
 * Method:    doStringFromIID
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_jacob_com_COMUtils_doStringFromIID
  (JNIEnv *env, jclass, jstring  _in)
{
  CLSID clsid;
  HRESULT hr;
  OLECHAR bsProgId[MAX_CHARS]; 	
  OLECHAR StringBase[MAX_CHARS]; 	
  LPOLESTR lpsz = StringBase;
  jstring ret;
  // Now, try to find an IDispatch interface for progid
  copyStringChars(env,_in,bsProgId);

  // Now, try to find an IDispatch interface for progid
  hr = CLSIDFromString(bsProgId, &clsid);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't get object clsid from progid", hr);
    return NULL;
  }

  hr = StringFromGUID2(clsid,lpsz,MAX_CHARS);
  if (FAILED(hr)) {
    ThrowComFail(env, "Can't convert GUID to String", hr);
    return NULL;
  }

    // Handle string...
   ret = env->NewString(lpsz,oleStrLen(lpsz));
  return(ret);
}


}
