/*
 * Copyright (c) 2001 Steven Lewis
 * mailto:smlewis@lordjoe.com All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in 
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Redistributions in any form must be accompanied by information on
 *    how to obtain complete source code for the JACOB software.
 *
 * Redistribution of the JACOB software is not permitted as part of any
 * commercial product that is targeted primarily at Java developers.
 * Such products include, but are not limited to: Java virtual machines,
 * integrated development environments, code libraries, and application
 * server products. Licensing terms for such distribution may be
 * obtained from the copyright holder.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package com.jacob.com;

/**
 * This class is a Wrapper which acts through a DIspatch
 * Composition is used rather than inheritance because it allows
 * subclassing without complex native methods
 */
public abstract class WDispatch implements DispatchConstants
{
    public static Variant[] NULL_VARIANT_ARRAY = {} ;
    
    private Dispatch m_Dispatch;

  static {
    System.runFinalizersOnExit(true);
  }
  public WDispatch(String progid)
  {
    setDispatch(new Dispatch(progid));
  }
  
  public WDispatch(int progid)
  {
    setDispatch(new Dispatch(progid));
  }
  
  public WDispatch(Dispatch theObj) {
       setDispatch(theObj);
  } 
  
  public WDispatch(WDispatch theObj) {
       setDispatch(theObj);
  } 
 
  
  public WDispatch(Object theObj) {
       setDispatch(dispatchFromObj(theObj));
  } 
  
  public abstract String getClsId();
  
  
  
  protected void setDispatch(WDispatch in) {
    setDispatch(getCompatibleDispatch(in,this));
  } 
  
  protected void setDispatch(Dispatch d) 
  {
      if(d.getDispatch() == 0)
            m_Dispatch = d; // break here - should be an exception
      m_Dispatch = d;
  }
  
  protected static Dispatch dispatchFromObj(Object theObj)
  {
      if(theObj instanceof Dispatch) {
         return((Dispatch)theObj);
      }
      if(theObj instanceof Variant) {
         return(((Variant)theObj).toDispatch());
      }
      throw new IllegalArgumentException("Wrong class to build dispatch "  + getClassName(theObj));
      
  }
  
    public void addListener(String s, Object eventlistener)
    {
        String s1 = eventlistener.getClass().getSuperclass().getName();
        int i = s1.lastIndexOf(".");
        String s2 = s1.substring(0, i);
        throw new UnsupportedOperationException("Fix This");
    //    pCEventListener = IDispatch_addListener(IDispatch, s, eventlistener, s2);
    }

    public void removeListener(String s, Object eventlistener)
    {
        throw new UnsupportedOperationException("Fix This");
  //      IDispatch_removeListener(pCEventListener);
    }

    public static String getClassName(Object obj)
    {
        return obj.getClass().getName();
    }

    public Variant invoke_method(Variant[] args, int DispID, short wFlags)
    {
        int err = 0;
        int[] uArgErr = { err };
        Variant ret = Dispatch.invokev(m_Dispatch, DispID, wFlags,args , uArgErr);
        return(ret);
    }

    public void invoke_method_void(Variant[] args, int DispID, short wFlags)
    {
        int err = 0;
        int[] uArgErr = { err };
        Dispatch.invokev(m_Dispatch, DispID, wFlags,args, uArgErr);
    }
    
 
  public Variant invoke(String name, Variant[] args)
  {
    return Dispatch.callN(m_Dispatch, name, args);
  }

  public Variant getProperty(String name)
  {
    return Dispatch.get(m_Dispatch, name);
  }

  public void setProperty(String name, Variant arg)
  {
    Dispatch.put(m_Dispatch, name, arg);
  }
  
  public Dispatch getDispatch()
  {
      return(m_Dispatch);
  }
  
  protected static Dispatch getCompatibleDispatch(WDispatch in,WDispatch out)
  {
       String Oldclsid = in.getClsId();
       String clsid = out.getClsId();
       Dispatch d = in.getDispatch();
       if(!Oldclsid.equals(clsid))
            d = d.QueryInterface(clsid);
       return(d);
  }
}
